/*
 * xman - X window system manual page display program.
 *
 * $XConsortium: version.h,v 1.9 91/06/08 18:15:37 rws Exp $
 *
 * Copyright 1987, 1988 Massachusetts Institute of Technology
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * Author:    Chris D. Peterson, MIT Project Athena
 * Created:   July 18, 1988 
 */

#ifdef MOTIF
#define XMAN_VERSION "Motif Xman Version 3.1.5 - X11R5."
#else
#define XMAN_VERSION "Xman Version 3.1.5 - X11R5."
#endif

