#ifdef MOTIF
#ifndef MENUSUPPORT_H
#define MENUSUPPORT_H
/****************************************************************************
 *         This example code is from the book:
 *
 *         The X Window System Programming and Applications with Xt,
 *	     OSF/Motif, second edition
 *         by
 *           Douglas Young
 *
 *         Copyright 1993, 1994 by Prentice Hall
 *         All Rights Reserved
 *
 *  Permission to use, copy, modify, and distribute this software for
 *  any purpose except publication and without fee is hereby granted, provided
 *  that the above copyright notice appear in all copies of the software.
 **************************************************************************/


#include <Xm/Xm.h>

typedef enum _menu_type { POPUP, MENUBAR, OPTION, PULLDOWN,
RADIOPULLDOWN, BUTTON, LABEL, TOGGLE, SEPARATOR } MenuType;

typedef struct _menu_desc {
   MenuType               type;
   char*                  name;           /* name of the button */
   XtCallbackProc         func;           /* Callback to be invoked */
   XtPointer              data;           /* Data for the callback */
   struct _menu_desc     *subMenu;       /* data for submenu of this
button */
} MenuDescription;

Widget createMenu(MenuType menuType,
		  char *name,
		  Widget parent,
		  MenuDescription *desc,
		  XtPointer defaultClientData);


#endif
#endif


